// This file is part of the Dynpeak toolbox
// Copyright (C) 2012 - INRIA - Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
function SamplingEffectGUI()
  margin_x     = 5;      // Horizontal margin between each elements
  margin_y     = 5;      // Vertical margin between each elements
  button_w     = 90;
  button_h     = 20;
  info_h       = 100;
  label_h      = 20;
  label_w      = 170;
  popup_w      = 130;
  popup_h      = label_h;

  frame_h     = 14*label_h+13*margin_y+button_h+info_h;
  frame_w     = 380;
  axes_w       = 2*margin_x+frame_w;
  axes_h       = 2*margin_y+frame_h;// Frame height
  defaultfont  = "arial"; // Default Font
  
  dlg_id=100001;
  dlg = scf(dlg_id);clf;

  // Remove Scilab graphics menus & toolbar
  //  drawlater (bug)
  delmenu(dlg.figure_id, gettext("&File"));
  delmenu(dlg.figure_id, gettext("&Tools"));
  delmenu(dlg.figure_id, gettext("&Edit"));
  delmenu(dlg.figure_id, gettext("&?"));
  toolbar(dlg.figure_id, "off");
  dlg.axes_size       = [axes_w axes_h];

  
  dlg.background      = addcolor([0.8 0.8 0.8]);
  dlg.figure_name     = _("Sampling effect demo");
  ax=dlg.children;
  ax.background= dlg.background ;
  
  gui=uicontrol( ...
      "parent"              , dlg,...
      "style"               , "frame",...
      "units"               , "pixels",...
      "position"            , [margin_x 0 frame_w frame_h],...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");

  y=frame_h-margin_y-2*label_h;
  x1=margin_x;  
  
  uicontrol( ...
      "parent"              , gui,...
      "style"               , "text",...
      "string"              , _("Sampling effect demo"),...
      "units"               , "pixels",...
      "position"            , [x1 y frame_w-2*margin_x 2*label_h],....
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 24,...
      "fontweight"          , "bold", ...
      "horizontalalignment" , "center", ...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");
  
   y=y-info_h-margin_y;
  txt=["<html>"
       "This demo illustate the effect of the initial sample date,<br>"
       "&#160;&#160 of the uncertainty on the sample dates and on the measured<br>" 
       "&#160;&#160 LH level when the sampling period is quite high relatively<br>"
       "&#160;&#160 to the LH signal period.<br>"
       "The demo is based on generated synthetic LH signals which <br>"
       "&#160;&#160 can have constant or varying amplitude and period.<br>"
       "</html>"];
  uicontrol( ...
      "parent"              , gui,...
      "style"               , "text",...
      "string"              , strcat(txt),...
      "units"               , "pixels",...
      "position"            , [x1 y frame_w-2*margin_x info_h],....
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");
 y=y-label_h-margin_y;
  uicontrol( ...
      "parent"              , gui,...
      "style"               , "text",...
      "string"              , _("Synthetic LH signal"),...
      "units"               , "pixels",...
      "position"            , [x1 y frame_w-2*margin_x label_h],....
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "fontweight"          , "bold", ...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");
  
  y=y-2*label_h-margin_y;
  u=uicontrol( ...
      "parent"              , gui,...
      "style"               , "text",...
      "string"              , "",...
      "units"               , "pixels",...
      "position"            , [x1 y frame_w 2*label_h],....
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "fontweight"          , "bold", ...
      "horizontalalignment" , "center", ...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");
  u.string="$\frac{d LH}{d t} = M_{spike}(t) e^{-k_{hl} (t-floor(t/P_{spike}(t)) P_{spike}(t))}-6 LH(T)$";

  y=y-label_h-margin_y;
  uicontrol( ...
      "parent"              , gui,...
      "style"               , "text",...
      "string"              , "$M_{spike}(t)$",...
      "units"               , "pixels",...
      "position"            , [x1 y label_w label_h],....
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "fontweight"          , "bold", ...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");
  x2=x1+label_w+margin_x;
  
  M=uicontrol( ...
      "parent"              , gui,...
      "style"               , "popupmenu",...
      "string"              , ["15 - 0.0087t";"15 ng/(ml.min)"],...
      "value"               , 2,...
      "units"               , "pixels",...
      "position"            , [x2 y popup_w popup_h],....
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");

  y=y-label_h-margin_y;
  uicontrol( ...
      "parent"              , gui,...
      "style"               , "text",...
      "string"              , "$P_{spike}(t)$",...
      "units"               , "pixels",...
      "position"            , [x1 y label_w label_h],....
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "fontweight"          , "bold", ...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");
  x2=x1+label_w+margin_x;
  
  P=uicontrol( ...
      "parent"              , gui,...
      "style"               , "popupmenu",...
      "string"              , ["60 min";"60-t/50 min"],...
      "value"               , 2,...
      "units"               , "pixels",...
      "position"            , [x2 y popup_w popup_h],...
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");

  y=y-2*label_h-margin_y;
  uicontrol( ...
      "parent"              , gui,...
      "style"               , "text",...
      "string"              , _("Sampling parameters"),...
      "units"               , "pixels",...
      "position"            , [x1 y frame_w-2*margin_x label_h],....
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "fontweight"          , "bold", ...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");  
  
  y=y-label_h-margin_y;
  uicontrol( ...
      "parent"              , gui,...
      "style"               , "text",...
      "string"              , "First sample date",...
      "units"               , "pixels",...
      "position"            , [x1 y label_w label_h],....
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");
  x2=x1+label_w+margin_x;
  
  r=uicontrol( ...
      "parent"              , gui,...
      "style"               , "popupmenu",...
      "string"              , ["1 min";"2 min";"4 min"],...
      "value"               , 3,...
      "units"               , "pixels",...
      "position"            , [x2 y popup_w popup_h],...
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");

  y=y-label_h-margin_y;
  uicontrol( ...
      "parent"              , gui,...
      "style"               , "text",...
      "string"              , "Uncertainty on sample date",...
      "units"               , "pixels",...
      "position"            , [x1 y label_w label_h],....
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");
  x2=x1+label_w+margin_x;
  
  f=uicontrol( ...
      "parent"              , gui,...
      "style"               , "popupmenu",...
      "string"              , ["0.5 min";"1.5 min";"2 min"],...
      "value"               , 2,...
      "units"               , "pixels",...
      "position"            , [x2 y popup_w popup_h],...
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");

  y=y-label_h-margin_y;
  uicontrol( ...
      "parent"              , gui,...
      "style"               , "text",...
      "string"              , "Uncertainty on LH measure",...
      "units"               , "pixels",...
      "position"            , [x1 y label_w label_h],....
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");
  x2=x1+label_w+margin_x;
  
  b=uicontrol( ...
      "parent"              , gui,...
      "style"               , "popupmenu",...
      "string"              , ["2%";"5%";"10%"],...
      "value"               , 2,...
      "units"               , "pixels",...
      "position"            , [x2 y popup_w popup_h],...
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");
  y=y-2*label_h-margin_y;
  uicontrol( ...
      "parent"              , gui,...
      "relief"              , "groove",...
      "style"               , "pushbutton",...
      "string"              , _("Show"),...
      "units"               , "pixels",...
      "position"            , [x1 y button_w button_h],....
      "fontname"            , defaultfont,...
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "fontweight"          , "bold", ...
      "horizontalalignment" , "center", ...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on",...
      "userdata"            , [M P r f b],...
      "callback"            , "ShowSamplingEffect()")

endfunction
function ShowSamplingEffect(Mspike,Pspike,r,f,b)
//Mspike : a character string that contains an expression which computes the
//         spike amplitude as a function of time given in minutes
//         (example: "15-t*8.7e-3")
  
//Pspike : a character string that contains an expression which computes the
//         spike periode as a function of time given in minutes
//         (example: "100-t/30")
//r      : initial time (in minute of the first sample)
//f      : sampling time variability
//b      : uncertainty on the measured sampled signa values.
  Calling=[]
  if argn(2)==0 then
    Calling=gcbo
    H=Calling.userdata;//[M P r f b]
    
    select H(1).value
    case 1 then Mspike="15 - 0.0087*t";
    case 2 then Mspike="15";
    end
    select H(2).value
    case 1 then Pspike="60";
    case 2 then Pspike="60-t/50"
    end
    select H(3).value
    case 1 then r=1;
    case 2 then r=2;
    case 3 then r=4;
    end
    select H(4).value
    case 1 then f=0.5;
    case 2 then f=1.5;
    case 3 then f=2;
    end
    select H(5).value
    case 1 then b=0.02;
    case 2 then b=0.05;
    case 3 then b=0.1;
    end
    Calling.enable="off";
  end
  deff("A=mspike(t)","A="+Mspike)
  deff("P=pspike(t)","P="+Pspike)
  
  k_hl=0.03465735903; //stiffness coefficient 
  Alpha=6;// instantaneous LH clearance rate (1/min)

  //Integrate theoritical (continuuous time) model
  Tmax=1500;
  dt=0.01;//observation step for theoritical model
  T=0:dt:Tmax;//observation dates for theoritical model
  LH=Theoritical_LH(T,mspike,pspike,k_hl,Alpha);
  LHmax=max(LH)
  tsampling=10;
  [Ts,LHs]=NoisedSampled_LH(T,LH, tsampling,r,f,b);
  fig=scf(max(winsid())+1);clf;drawlater()
  fig.axes_size = [680,680];
  fig.figure_name="Effect of sampling process";
  //Title
  axt=gca(); 
  axt.axes_bounds=[0 0 1 1/10];
  tit=["Effect of sampling process"
       "Mspike="+Mspike+", Pspike="+Pspike+"min, r="+string(r)+"min, ts=10+/-"+string(f)+"min, b="+string(b*100)+"%"];
  xstringb(0,0,tit,1,1);s=gce();
  s.font_style=8;s.font_size=2;

  dy=(1-1/9)/3;
  h=0.8*dy
  y=1/9;
  //Synthetic Signal
  ax=newaxes(); 
  ax.axes_bounds=[0 y 1 h];
  ax.axes_visible="on";
  ax.data_bounds=[0 0;Tmax LHmax];
  ax.x_label.text="Time (min)";
  ax.x_label.position=[Tmax,0];
  ax.y_label.text="LH (ng/ml)";
  ax.title.text="Synthetic signal";
  ax.title.font_style=8;
  ax.title.font_size=2;
  ax.x_ticks.labels=emptystr(ax.x_ticks.labels);
  xpoly(T,LH);p=gce();
  p.foreground=3;
  p.clip_state="clipgrf";
  //Sampled Signal
  ax=newaxes(); 
  y=y+dy;
  ax.axes_bounds=[0 y 1 h];
  ax.axes_visible="on";
  ax.data_bounds=[0 0;Tmax LHmax];
  ax.margins(4)=0.2;
  ax.x_label.text="Time (min)";
  ax.x_label.position=[Tmax,0];
  ax.y_label.text="LH (ng/ml)";
  ax.title.text="Sampled signal";
  ax.title.font_style=8;
  ax.title.font_size=2;
  ax.x_ticks.labels=emptystr(ax.x_ticks.labels);
  xpoly(Ts,LHs);p=gce();
  p.foreground=2;
  p.mark_mode="on";
  p.mark_size_unit="point";
  p.mark_size=2;
  p.foreground=2;
  p.mark_foreground=2;
  p.clip_state="clipgrf";
  
  //Enveloppe of the synthetic signal
  peakind=find(LH(1:$-2)<LH(2:$-1)&LH(3:$)<LH(2:$-1))+1;
  xpoly(T(peakind),LH(peakind));p=gce()
  p.foreground=3;
   p.line_style=2;
  p.clip_state="clipgrf";
  peakinds=find(LHs(1:$-2)<LHs(2:$-1)&LHs(3:$)<LHs(2:$-1))+1;
  xpoly(Ts(peakinds),LHs(peakinds));p=gce()
  p.foreground=2;
  p.line_style=2;
  p.clip_state="clipgrf";

  L=legend(["Sampled signal";"Synthetic signal peaks";"Sample signal peaks"],"lower_caption",%f);
  //IPI
  IPI=T(peakind(2:$))-T(peakind(1:$-1));
  IPIs=Ts(peakinds(2:$))-Ts(peakinds(1:$-1));
  IPImax=max(max(IPI),max(IPIs))
  IPImin=min(min(IPI),min(IPIs))
  ax=newaxes(); 
  y=y+dy;
  ax.axes_bounds=[0 y 1 h];
  ax.axes_visible="on";
  ax.data_bounds=[0 IPImin;Tmax IPImax];
  ax.margins(4)=0.2;
  ax.grid=[1 1]*color('gray');
  ax.x_label.text="Time (min)";
  ax.x_label.position=[Tmax,0];
  ax.y_label.text="IPI (min)";
  ax.title.text="Inter Peak Intervals";
  ax.title.font_style=8;
  ax.title.font_size=2;

  xpoly(T(peakind(1:$-1)),IPI);p=gce()
  p.foreground=3;
  p.clip_state="clipgrf";
  xpoly(Ts(peakinds(1:$-1)),IPIs);p=gce()
  p.foreground=2;
  p.clip_state="clipgrf";
  L=legend(["Synthetic signal IPI";"Sample signal IPI"],"lower_caption",%f);
 drawnow()
 if Calling<>[] then
    Calling.enable="on";
  end
endfunction
