// This file is part of the Dynpeak toolbox
// Copyright (C) 2012 - INRIA  - Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
function [Ts,LHs]=NoisedSampled_LH(T,LH,tstep,r,f,CV)
//computes Noised LH signal sampled given the theoritical LH signal
//   sampled at high frequency.
  
//T           : theoritical LHp signal observation dates (sampled with high frequency)
//LH          : theoritical LHp signal
//tstep       : sampling time in number of minutes
//r           : date of the first sample in min
//f           : standard deviation of the error on the measure date
//CV          : function which computes the measurement uncertainty for a given
//              LH level
  fname="NoisedSampled_LH"
  if type(T)<>1|~isreal(T)|or(T<0) then
    error(msprintf(_("%s: Wrong type for argument %d: Real positive vector expected.\n"),fname,1))
  end
  if T(1)<0|or(diff(T)<=0) then
    error(msprintf(_("%s: Wrong value for argument %d: increasing order expected.\n"),fname,1))
  end
  if type(LH)<>1|~isreal(LH) then
    error(msprintf(_("%s: Wrong type for argument %d: Real  vector expected.\n"),fname,2))
  end
  if size(LH,'*')<>size(T,'*') then
    error(msprintf(_("%s: Incompatible input arguments #%d and #%d: Same sizes expected.\n"),fname,1,2))
  end
  dt=T(2)-T(1)
  if type(tstep)<>1|size(tstep,"*")<>1|~isreal(tstep) then
    error(msprintf(_("%s: Wrong type for argument %d: Real scalar expected.\n"),fname,3))
  end
  if tstep<dt then
    error(msprintf(_("%s: Wrong value for input argument #%d: Must be > %g.\n"),fname,3,dt))
  end
  if type(r)<>1|size(r,"*")<>1|~isreal(r)|r<0 then
    error(msprintf(_("%s: Wrong type for argument %d: Real non negative scalar expected.\n"),fname,4))
  end
  if type(f)<>1|size(f,"*")<>1|~isreal(f)|f<0 then
    error(msprintf(_("%s: Wrong type for argument %d: Real non negative scalar expected.\n"),fname,5))
  end
  if and(type(CV)<>[1 13])|...
        (type(CV)==1&(size(CV,"*")<>1|~isreal(CV)|CV<0)) then
    error(msprintf(_("%s: Wrong type for argument %d: Real non negative scalar or a function expected.\n"),fname,6))
  end
  rand("normal")
  Tr=2.2;//Distribution truncating parameter, avoids large deviation effect
  n=size(LH,'*')
  dt=T(2)-T(1)
  c=tstep/dt; // Sampling every tstep minutes approximately
  rp=r/dt; // First sample taken at r minutes
  if type(CV)==1 then
    cv=CV*ones(LH)
  else
    cv=CV(LH)
  end
  //number of samples
  ns=(n-c-rp-1)/c
  LHs=zeros(1,ns)
  Ts=zeros(1,ns)
  j=1;
  for i=1:c:(n-c-rp-1)
    u=rand();while abs(u)>Tr,u=rand(); end
    e=floor(f/dt*rand());
    if i+e<0 then ii=i-e+rp,else ii=i+e+rp;end
    ii=min(ii,n)
    LHs(j)=LH(ii)*(1+cv(ii)*u);
    Ts(j)=T(i+rp)
    j=j+1;
  end
endfunction
