// This file is part of the Dynpeak toolbox
// Copyright (C) 2012 - INRIA - Qinghua ZHANG - Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
function LHp=Theoritical_LH(T,Amplitude,Period,k_lh,Alpha)
// T         : observation time instants
// LHp       : computed  measured LH level at the requested time instants.
// Amplitude : pituitary gland LH spike amplitude 
// Period    : pituitary gland LH spike period 
// k_hl      : stiffness coefficient (log(2)/tau_hl, where tau_hl is the spike half-life)
// Alpha     : represents the instantaneous LH clearance rate from the blood.
  fname="Theoritical_LH";
  if type(T)<>1|~isreal(T)|or(T<0) then
    error(msprintf(_("%s: Wrong type for argument %d: Real positive vector expected.\n"),fname,1))
  end
  if T(1)<0|or(diff(T)<=0) then
    error(msprintf(_("%s: Wrong value for argument %d: increasing order expected.\n"),fname,1))
  end
  if type(Amplitude)<>13 then
    error(msprintf(_("%s: Wrong type for argument %d: a function expected.\n"),fname,2))
  end
  if type(Period)<>13 then
    error(msprintf(_("%s: Wrong type for argument %d: a function expected.\n"),fname,3))
  end
  if type(k_lh)<>1|size(k_lh,"*")<>1|~isreal(k_lh)|k_lh<0 then
    error(msprintf(_("%s: Wrong type for argument %d: Real non negative scalar expected.\n"),fname,4))
  end
  if type(Alpha)<>1|size(Alpha,"*")<>1|~isreal(Alpha)|Alpha<0 then
    error(msprintf(_("%s: Wrong type for argument %d: Real non negative scalar expected.\n"),fname,5))
  end
  
  LH0 = 0;
  LHp = ode(LH0,0,T,list(Theoritical_LH_Dynamics,Amplitude,Period,k_hl,Alpha));
endfunction

function dLHp = Theoritical_LH_Dynamics(t,LHp,Amplitude,Period,k_hl,Alpha)
// t         : current time
// LHp       : current measured LH level in the jugular blood.
// Amplitude : pituitary gland LH spike amplitude 
// Period    : pituitary gland LH spike period 
// k_hl      : stiffness coefficient (log(2)/tau_hl, where tau_hl is the spike half-life)
// Alpha     : represents the instantaneous LH clearance rate from the blood.
  P=Period(t)
  A=Amplitude(t)
  dLHp = A.*exp(-k_hl*(t-floor(t./P).*P))-Alpha*LHp;
endfunction
