// This file is part of the Dynpeak toolbox
// Copyright (C) 2012 - INRIA - Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
function Text=dynpeak_create_xml(D)
// D fields : selected,history, T, LH, Alpha, detectionparams,indpeaks
  
  T=D.T
  LH=D.LH
  p=D.detectionparams
  indpeaks=D.indpeaks
  imposedpeaks=p.imposedpeaks
  ignoredpoints=p.ignoredpoints
  N=size(T,'*')
 
  IPI=diff(T(indpeaks))
  
  //define the Cell styles
  Heading= XMLSpreadsheet_Style("Heading",XMLSpreadsheet_Font("Bold",1,"Size",12));
  ce1    = XMLSpreadsheet_Style("ce1",XMLSpreadsheet_Font("Size",10));
  ce1r   = XMLSpreadsheet_Style("ce1",XMLSpreadsheet_Font("Size",10,"Color","ff0000"))
  ce1g   = XMLSpreadsheet_Style("ce1",XMLSpreadsheet_Font("Size",10,"Color","00ff00"))

  cep    = XMLSpreadsheet_Style("cep",[XMLSpreadsheet_Font("Size",10);
                                       XMLSpreadsheet_NFormat("Format","Percent")]);
  ce2    = XMLSpreadsheet_Style("ce2",[XMLSpreadsheet_Font("Size",10);
                                      XMLSpreadsheet_Interior("Color","#ff0000","Pattern","Solid")]);
  Styles=XMLSpreadsheet_Styles([Heading;ce1;ce1r;ce1g;ce2;cep]);
  
  //create the cell array of given data
  nrows=N+1;ncols=6
  Cells=XMLSpreadsheet_Cell(%nan(ones(nrows,ncols)),'ce1')
  //fill in the initial data

  Cells(1,1:2)=XMLSpreadsheet_Cell([_("Time"),_("LH")],"Heading")
  Cells(2:nrows,1:2)=XMLSpreadsheet_Cell([T,LH],'ce1')
  if ignoredpoints<>[] then
    Cells(ignoredpoints,1:2)=XMLSpreadsheet_Cell([T(ignoredpoints),LH(ignoredpoints)],'ce1r')
  end
  if imposedpeaks<>[] then
    Cells(imposedpeaks,1:2)=XMLSpreadsheet_Cell([T(imposedpeaks),LH(imposedpeaks)],'ce1r')
  end
  
  //fill in the detected peak data
  np=size(indpeaks,'*')
  Cells(1,4:6)=XMLSpreadsheet_Cell([_("Index"),_("Time"),_("IPI")],"Heading")
  Cells(2:np+1,4:6)=XMLSpreadsheet_Cell([indpeaks,T(indpeaks),[%nan;IPI]],'ce1');

  
  //form a table with cell array
  Table=XMLSpreadsheet_Table(Cells,"ta1");
  //form a sheet with table
  S1=XMLSpreadsheet_Worksheet(Table,_("Data and result"))



  uncertainty=p.uncertainty
  m=size(uncertainty,2)
  Cells=XMLSpreadsheet_Cell(%nan(ones(6,m+1)),'ce1')
  Cells(:,1)=XMLSpreadsheet_Cell([_("Detection threshold");
                    _("Variation coefficient range (min)");
                    _("Variation coefficient CV ");
                    _("Nominal peak period (min)");
                    _("Global relative threshold")
                    _("Three point threshold")],"Heading")
  Cells(1,2)=XMLSpreadsheet_Cell(p.localabstol,"ce1")
  Cells(2,2:m+1)=XMLSpreadsheet_Cell(uncertainty(1,:),"ce1")
  Cells(3,2:m+1)=XMLSpreadsheet_Cell(uncertainty(2,:)*100,"cep")
  Cells(4,2)=XMLSpreadsheet_Cell(p.period,"ce1");
  Cells(5,2)=XMLSpreadsheet_Cell(p.globalreltol,"cep");
  Cells(6,2)=XMLSpreadsheet_Cell(p.threepointtol,"cep");
  
  S2=XMLSpreadsheet_Worksheet(XMLSpreadsheet_Table(Cells,"ta1"),_("Parameters"))

   
  Text=XMLSpreadsheet_Document(Styles,[S1;S2],"Dynpeak - Scilab");
endfunction
function F=XMLSpreadsheet_NFormat(varargin)
  valid="Format"
  F="<NumberFormat"
  for k=1:2:size(varargin)
    F=F+" ss:"+varargin(k)+"="""+string(varargin(k+1))+""""
  end
  F=F+"/>"
endfunction

function F=XMLSpreadsheet_Font(varargin)
  valid=["Bold","Color","FontName","Italic","Outline","Shadow" "Size","StrikeThrough","Underline","VerticalAlign"]
  F="<Font"
  for k=1:2:size(varargin)
    F=F+" ss:"+varargin(k)+"="""+string(varargin(k+1))+""""
  end
  F=F+"/>"
endfunction

function F=XMLSpreadsheet_Interior(varargin)
  valid=["Color","Pattern","PatternColor"]
  F="<Interior"
  for k=1:2:size(varargin)
    F=F+" ss:"+varargin(k)+"="""+string(varargin(k+1))+""""
  end
  F=F+"/>"
endfunction

function S=XMLSpreadsheet_Style(id,style)
  S=["<Style ss:ID="""+id+""" ss:Name="""+id+""">"
     "  "+style
     "</Style>"]
endfunction

function S=XMLSpreadsheet_Styles(styles)
  S=["<Styles>"
     "  "+styles
     "</Styles>"];
endfunction
         
function M=XMLSpreadsheet_Cell(value,style)
//voir http://msdn.microsoft.com/en-us/library/aa140066%28v=office.10%29
  select type(value)
  case 1 then
    k1=find(isnan(value))
    k2=find(~isnan(value))
    value=string(value)
    value(k1)=""
    value(k2)="<Data ss:Type=""Number"">"+value(k2)+"</Data>"
    
    M="<Cell ss:StyleID="""+style+""">"+value+"</Cell>"
  case 4 then
    M="<Cell ss:StyleID="""+style+"""><Data ss:Type=""Boolean"">"+string(bool2s(value))+"</Data></Cell>"
  case 10 then
    M="<Cell ss:StyleID="""+style+"""><Data ss:Type=""String"">"+value+"</Data></Cell>"
  else
    error(msprintf(_("%s : unexpected type for argument %d.\n"),"XMLSpreadsheet_cell",2))
  end
endfunction
function T=XMLSpreadsheet_Table(M,style)
  [m,n]=size(M)
  T=["<Table ss:StyleID="""+style+""" ss:ExpandedColumnCount="""+string(n)+""" ss:ExpandedRowCount="""+string(m)+""">";
     "  "+matrix(["<Row>"+emptystr(m,1),"  "+M,"</Row>"+emptystr(m,1)]',-1,1)
     "</Table>"]
endfunction
function T=XMLSpreadsheet_Worksheet(content,name)
  T=["<Worksheet ss:Name="""+name+""">"
     "  "+content
     "  <WorksheetOptions xmlns=""urn:schemas-microsoft-com:office:excel"">"; 
     "    <ProtectObjects>False</ProtectObjects>"; 
     "    <ProtectScenarios>False</ProtectScenarios>"; 
     "  </WorksheetOptions>"; 
     "</Worksheet>"]
endfunction

function T=XMLSpreadsheet_Document(styles,content,author,date)
//voir http://msdn.microsoft.com/en-us/library/aa140066%28v=office.10%29
  T=["<?xml version=""1.0""  encoding=""UTF-8""?>"; 
     "<?mso-application progid=""Excel.Sheet""?>"; 
     "<Workbook xmlns=""urn:schemas-microsoft-com:office:spreadsheet"""; 
     "   xmlns:o=""urn:schemas-microsoft-com:office:office"""; 
     "   xmlns:x=""urn:schemas-microsoft-com:office:excel"""; 
     "   xmlns:ss=""urn:schemas-microsoft-com:office:spreadsheet"""; 
     "   xmlns:html=""http://www.w3.org/TR/REC-html40"">"; 
     "   <DocumentProperties xmlns=""urn:schemas-microsoft-com:office:office"">"; 
     "     <Author>"""+author+"""</Author>"; 
     "     <LastAuthor>"""+author+"""</LastAuthor>"; 
     "     <Version>12.00</Version>"; 
     "   </DocumentProperties>"; 
     "   <ExcelWorkbook xmlns=""urn:schemas-microsoft-com:office:excel"">"; 
     "     <ProtectStructure>False</ProtectStructure>"; 
     "     <ProtectWindows>False</ProtectWindows>"; 
     "   </ExcelWorkbook>";
     "  "+styles
     "  "+content
     "</Workbook>"];
endfunction



     
