// This file is part of the Dynpeak toolbox
// Copyright (C) 2012 - INRIA - Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
function dynpeak_eventhandler(win,x,y,ibut)
  if and(ibut<>[0 2 3 5 1122]) then return,end
  fig=get_figure_handle(win);
  fig.event_handler_enable = "off";
  fig_ud=fig.user_data
  params=fig_ud.dynpeak;
  peak_alg_pars=params.detectionparams
  T=params.T;
  n=size(T,'*')
  sig=params.LH,
  Ax=fig.children(fig.children.type=="Axes")
  ax1=Ax(2);
  ax2=Ax(1)
  ax=getAxes([x,y],fig);
  sca(ax);
  if or(ibut==[0 3]) then //left click
    fp1=ax1.children(1);//handle on the imposed peaks entity
    ip1=ax1.children(2);//handle on the ignored point entity
    b1=ax1.children(3);//handle on the peak locations polyline (barplot)
    i1=ax1.children(4);//handle on the uncertainty bars
    p1=ax1.children(5);//handle on the signal polyline
    m1=ax1.children(6);//handle on the selected polyline
    dt1=ax1.children(7);//handle on the detection threshold line
    s1=ax1.children(8);//handle on the selection rectangle
    
    thigh=ax2.children(1); //high tunnel boundary
    tlow=ax2.children(2);  //low tunnel boundary
    tmed=ax2.children(3);  //tunnel mean line
    IPI=ax2.children(4); //IPI curve
    m2=ax2.children(5); //handle on the selected IPI point marker
    
    [x,y]=xchange(x,y,'i2f');
    if ax==ax1 then //first plot clicked
      dT=(T($)-T(1))/50;
      //check for selection boundary
      sel=peak_alg_pars.selection;
      if sel==[] then sel=[1;n],end
      d1=s1.data
      t=abs(x-T(sel))
      rep(3)=0;
      if or(t<dT) then
        if t(1)<dT then //left boundary
          while rep(3)<>-5
            rep=xgetmouse([%t,%t]);
            dx=x-rep(1);
            x=rep(1);
            s1.data([1 3])=[x max(1,s1.data(3)+dx)];
          end
          [m,sel(1)]=min(abs(T-x));
        elseif t(2)<dT then //right boundary
          while rep(3)<>-5
            rep=xgetmouse([%t,%t]);
            x=rep(1)
            s1.data(3)=max(1,x-s1.data(1));
          end
          [m,sel(2)]=min(abs(T-x));
        end
        if sel(2)>sel(1) then
          xinfo("")
          peak_alg_pars.selection=sel
          fig_ud.dynpeak.detectionparams=peak_alg_pars;
          set(fig,"user_data",fig_ud)
          indpeaks = lhpeaks(sig,T,peak_alg_pars);
          dynpeak_update_graph(fig,T,sig,indpeaks);
        else
          xinfo(_("Invalid selection!"))
          s1.data=d1;
        end
      else //check for a signal point
        [m,k]=min(abs((p1.data(:,1)-x)));
        xinfo(msprintf("ind=%d,T(ind)=%g",k,p1.data(k,1)))
        m1.data=p1.data(k,:);
        m1.mark_mode='on';
        m1.visible="on";
        //[w,k2]=min(abs(IPI.data(:,1)-T(k)));
        [w,k2]=min(abs(IPI.data(:,1)-p1.data(k,1)));
        m2.data=IPI.data(k2,:);
        m2.visible="on";
        fig.user_data.dynpeak.selected=k;
      end
    else //second plot clicked
      [m,k]=min(abs((IPI.data(:,1)-x)));
      m1.visible="on";
      m2.visible="on";

      m1.data=[b1.data(k,:)
               b1.data(k,1),0
               b1.data(k+1,1),0
               b1.data(k+1,:)];
      m1.mark_mode='off';
      m2.data=IPI.data(k,:);
      fig.user_data.dynpeak.selected=k;
    end
    
  elseif or(ibut==[2 5 1122]) then //right click
    ks=fig_ud.dynpeak.selected
    h=fig_ud.dynpeak.history
    if ibut==1122 then //Ctrl-Z
      if size(h)<>0 then sel=_("Undo (Ctrl-Z)"); else sel=[];end
    else
      Choices=[];
      if ks<>[]&ax==ax1 then;
        sca(ax)
        [x,y]=xchange(x,y,'i2f');
        if abs(T(ks)-x)/T($)<0.01&abs(sig(ks)-y)/max(sig)<0.02 then
          if (or(ks==peak_alg_pars.ignoredpoints)) then
            Choices=[Choices;
                     _("Accept selected point");
                     _("Impose a peak at selected point")]

          else
            Choices=[Choices;
                     _("Ignore selected point");
                     _("Impose a peak at selected point")]
          end
        end
      end
      if size(h)<>0 then Choices=[Choices;_("Undo (Ctrl-Z)")];end
      if Choices<>[] then sel=PopupMenu(Choices);else sel=[]; end
    end
    
    select sel
    case _("Ignore selected point") then //Ignore selected point
      peak_alg_pars.ignoredpoints=unique([peak_alg_pars.ignoredpoints;ks])
      fig_ud.dynpeak.selected=[]
      fig_ud.dynpeak.detectionparams=peak_alg_pars
      fig_ud.dynpeak.history($+1)=[-1,ks,sig(ks),T(ks)];
      set(fig,"user_data",fig_ud)
      indpeaks = lhpeaks(sig,T,peak_alg_pars);
      dynpeak_update_graph(fig,T,sig,indpeaks);
    case _("Accept selected point") then //Ignore selected point
      peak_alg_pars.ignoredpoints(peak_alg_pars.ignoredpoints==ks)=[];
      fig_ud.dynpeak.detectionparams=peak_alg_pars
      fig_ud.dynpeak.history($+1)=[2,ks,sig(ks),T(ks)];
      set(fig,"user_data",fig_ud)
      indpeaks = lhpeaks(sig,T,peak_alg_pars);
      dynpeak_update_graph(fig,T,sig,indpeaks);
    case _("Impose a peak at selected point") then
      peak_alg_pars=params.detectionparams
      peak_alg_pars.imposedpeaks=unique([peak_alg_pars.imposedpeaks;ks])
      fig_ud.dynpeak.selected=[]
      fig_ud.dynpeak.detectionparams=peak_alg_pars
      fig_ud.dynpeak.history($+1)=[1,ks,sig(ks),T(ks)];
      set(fig,"user_data",fig_ud)
      indpeaks = lhpeaks(sig,T, peak_alg_pars);
      dynpeak_update_graph(fig,T,sig,indpeaks);
    case _("Undo (Ctrl-Z)")
      h=fig_ud.dynpeak.history
      if size(h)>0 then
        last=h($);
        ks=last(2)
        select last(1)
        case -1 then //undo IgnorePoint
          ignoredpoints=peak_alg_pars.ignoredpoints
          ignoredpoints(ignoredpoints==ks)=[]
          peak_alg_pars.ignoredpoints=ignoredpoints
        case 1 then //undo imposedpeaks
          imposedpeaks=peak_alg_pars.imposedpeaks
          imposedpeaks(imposedpeaks==ks)=[]
          peak_alg_pars.imposedpeaks=imposedpeaks
        case 2 then //undo Accept point
          peak_alg_pars.ignoredpoints=unique([peak_alg_pars.ignoredpoints;ks])
        end
        fig_ud.dynpeak.detectionparams=peak_alg_pars
        //update the history data structure
        h($)=null();
        fig_ud.dynpeak.history=h
         //recompute the peaks
        //disable selection
        fig_ud.dynpeak.selected=[];
        set(fig,"user_data",fig_ud)
        indpeaks = lhpeaks(sig,T, peak_alg_pars(:));
        dynpeak_update_graph(fig,T,sig,indpeaks);
      end
    end
  end
  fig.event_handler_enable = "on";
endfunction

function ax_handle=getAxes(pt,fig)
//Returns the axes handle where a point given in pixel lies in the
//current (or given) figure
// pt        : the vector or coordinates in pixel in the figure
// fig       : optionnal handle on the figure
// ax_handle : selected axes handle
  if argn(2)==1 then fig=gcf(),end
  axes=get(fig,'children');
  axes(axes.type=='uimenu')=[];
  if size(axes,'*')==1 then ax_handle=axes;return,end
  sz=fig.axes_size;
  for k=1:size(axes,'*')
    ax_handle=axes(k);
    xbounds=ax_handle.axes_bounds(:,1)*sz(1);
    ybounds=ax_handle.axes_bounds(:,2)*sz(2);
    if prod(xbounds-pt(1))<0&prod(ybounds-pt(2))<0 then break,end
  end
endfunction
function sel=PopupMenu(Choices)
  h = uicontextmenu();
  // Create its items
  for k=1:size(Choices,'*')
    uimenu("Parent", h, "label", Choices(k));
  end
  // Display and wait for a user answer
  sel = uiwait(h);
endfunction
