// This file is part of the Dynpeak toolbox
// Copyright (C) 2012 - INRIA -  Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
function  [debug_handles]=dynpeak_init_debug()
  global previous_peakind previous_Q
  previous_peakind=[];previous_Q=[];
  
  if debug_mode==1 then //debug with graphic interaction
    
    dfig=scf(max(winsid())+1);drawlater()
    ax=gca();
    ax.axes_bounds=[0 0 1 1/3];
    ax.margins(1)=0;
    xstring(0,0.9,"");s1=gce();s1.font_style=8;
    xstring(0.05,0,"");s=gce();  s.font_style=0;
    ax=newaxes();
    ax.axes_bounds=[0 1/3 1 2/3];
    plot(t,sig);
    xpoly(0,0,"marks");m1=gce();
    m1.mark_size_unit="point";
    m1.mark_size = 6;m1.mark_style=9;m1.mark_background=2;//blue current
    xpoly(0,0,"marks");m2=gce();
    m2.mark_size_unit="point";
    m2.visible='off';
    m2.mark_size = 6;m2.mark_style=9;m2.mark_background=3;//green added
    xpoly(0,0,"marks");m4=gce();
    m4.mark_size_unit="point";
    m4.visible='off';
    m4.mark_size = 6;m4.mark_style=9;m3.mark_background=18;//white unsure
    xpoly(0,0,"marks");m3=gce();
    m3.mark_size_unit="point";
    m3.visible='off';
    m3.mark_size = 10;m3.mark_style=2;m3.mark_background=5;m3.mark_foreground=5; //red cross deleted
    xpoly(0,0,"marks");m5=gce();
    m5.mark_size_unit="point";
    m5.visible='off';
    m5.mark_size = 6;m5.mark_style=9;m5.mark_background=7;; //yellow change status
    drawnow()
    debug_handles=[dfig,s1,s,m1 m2 m3 m4 m5];
  else
    debug_handles=[]
  end
endfunction
function  dynpeak_end_debug()
  clearglobal previous_peakind previous_Q
endfunction
function debug_mode=dynpeak_show_debug(label,peakind,Q)
  global previous_peakind previous_Q
  debug_mode
  if debug_mode==1 then
    curfig=gcf();
    scf(debug_handles(1))
    s1=debug_handles(2)
    s=debug_handles(3)
    m1=debug_handles(4) //current peaks
    m2=debug_handles(5) //added peaks
    m3=debug_handles(6) //deleted peaks
    m4=debug_handles(7) //uncertain peaks
    m5=debug_handles(8) //change status
    s1.text=label
  end
  indent=part(" ",1:5)
  info=label

  temp=[]
  if label=="Alternate position" then
    if km==[] then return;end
    for k=1:size(km,'*')
      temp=[temp;
            msprintf("k=%d, %s\n",peakind(km(k)),sci2exp(alt(k)))]
    end
    s.text=temp;
    mprintf("%s\n",[label;indent+temp])
  else
    [w,kprev,kcur]=intersect(previous_peakind,peakind)
    p=previous_peakind;p(kprev)=[];
    c=peakind;c(kcur)=[];
    if debug_mode==1 then
      m3.visible='off';
      m2.visible='off';
    end
    mprintf("%s \n",label)
    if p<>[] then 
      temp=msprintf("removed:%s \n",sci2exp(p',0))
      mprintf("%s\n",indent+temp)
      if debug_mode==1 then
        //highlight deleted peaks
        m3.visible='on';
        m3.data=[t(p) sig(p)];twinkle(m3);
        temp=msprintf("removed:%s \n",sci2exp(p',0))
        s.text=temp;
        [btn,w,w]=xclick();
        if or(btn(1)==[5 2]) then debug_mode=0,end
      end
    end
    if c<>[] then
      temp=msprintf("added  :%s \n",sci2exp(c',0))
      mprintf("%s\n",indent+temp)
      if debug_mode==1 then
        //highlight added peaks
        m2.visible='on';
        m2.data=[t(c) sig(c)]; twinkle(m2);
        s.text=temp
        [btn,w,w]=xclick();
        if or(btn(1)==[5 2]) then debug_mode=0,end
      end
    end
    kc=peakind(kcur(find(Q(kcur)<>previous_Q(kprev))))
    if kc<>[] then //status change
      temp=msprintf("status changed  \n")
      mprintf("%s\n",indent+temp)
      if debug_mode==1 then
        //highlight added peaks
        m5.visible='on';
        m5.data=[t(kc) sig(kc)]; twinkle(m5);
        s.text=temp
        [btn,w,w]=xclick();
        if or(btn(1)==[5 2]) then debug_mode=0,end
      end
    end 
    //result
    w=string([peakind Q]');
    txt=emptystr(2,1);
    for k=1:size(w,2),txt=txt+part(w(:,k),1:max(length((w(:,k)))))+" ";end
    txt='['+txt+']';
    temp=[msprintf("result :%s\n",txt(1))
          msprintf("     Q :%s\n",txt(2))]
   
    if debug_mode==1 then
      m3.visible='off';
      m4.visible='off';
      m2.visible='off';
      m5.visible='off';
      m1.data=[t(peakind) sig(peakind)];


      k=find(Q==0)
      if k<>[] then
        m4.visible='on';
        m4.data=[t(peakind(k)) sig(peakind(k))];
      else
        m4.visible='off';
      end
      s.text=temp;
      [btn,w,w]=xclick();
      if or(btn(1)==[5 2]) then  debug_mode=0,end
    end

    mprintf("%s\n",indent+temp)

    scf(curfig)

  end
  previous_peakind=peakind;
  previous_Q=Q;
endfunction
