// This file is part of the Dynpeak toolbox
// Copyright (C) 2012 - INRIA - Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
function dynpeak_update_graph(fig,T,sig,indpeaks)
  fig_ud=fig.user_data
  //load menus sub-functions
  fig_ud=fig.user_data
  fig_ud.dynpeak.indpeaks=indpeaks
  set(fig,"user_data",fig_ud)
  pars=fig_ud.dynpeak;
  Alpha=pars.Alpha;Beta=Alpha
  imposedpeaks=pars.detectionparams.imposedpeaks
  ignoredpoints=pars.detectionparams.ignoredpoints
  selection=pars.detectionparams.selection
  if selection==[] then selection=[1;size(T,'*')];end
  
  drawlater()
  T=T(:);sig=sig(:);
  Ax=fig.children(fig.children.type=="Axes")
  ax1=Ax(2);
  ax2=Ax(1)
 
  //Signal and peak locations graph
  fp1=ax1.children(1);//handle on the imposed peaks entity
  ip1=ax1.children(2);//handle on the ignored point entity
  b1=ax1.children(3);//handle on the peak locations polyline (barplot)
  i1=ax1.children(4);//handle on the incertitude bars
  p1=ax1.children(5);//handle on the signal polyline
  m1=ax1.children(6);//handle on the selected polyline
  dt1=ax1.children(7);//handle on the detection threshold line
  s1=ax1.children(8);//handle on the selection rectangle
  
  ax1.data_bounds(:,1)=[T(1);T($)];
  //update selection rectangle
  s1.data=[T(selection(1)),1.5*max(sig),T(selection(2))-T(selection(1)),2*max(sig)]

  //update detection threshold line

  dt1.data=[T(1) pars.detectionparams.detectionthreshold;
            T($) pars.detectionparams.detectionthreshold];

  //update signal polyline
  p1.data=[T,sig]; 
  
  //update uncertainty bars
  p=CV_LH(sig,pars.detectionparams.uncertainty)
  if or(p>0) then
    p=matrix(p,1,-1)
    i1.data=[matrix(ones(2,1)*T',-1,1) matrix([sig'-p;sig'+p],-1,1)]
    i1.segs_color=ones(T')*color('gray')
    i1.visible='on'
  else 
    i1.data=[0 0;0 0]
    i1.visible='off'
    i1.segs_color=color('gray')
  end
  
  //update  peak locations 
  loc=T(indpeaks(:))
  b1.data=[loc ax1.data_bounds(2,2)*1.5*ones(loc)];
  
  // hide seleted point marker
  m1.data=[];
  m1.visible="off";
  //update imposed peaks
  if imposedpeaks==[] then
    fp1.data=[]
  else
    fp1.data=[T(imposedpeaks) ax1.data_bounds(2,2)*1.5*ones(imposedpeaks)]
  end
   //update ignored points
  if ignoredpoints==[] then
    ip1.data=[]
  else
    ip1.data=[T(ignoredpoints) sig(ignoredpoints)]
  end
  //update IPI and tunnel graph
  thigh=ax2.children(1); //high tunnel boundary
  tlow=ax2.children(2);  //low tunnel boundary
  tmed=ax2.children(3);  //tunnel mean line
  IPI=ax2.children(4); //IPI curve
  m2=ax2.children(5); //handle on the selected IPI point marker
  ax2.data_bounds(:,1)=[T(1);T($)];
  
  //update tunnel
  y=diff(loc);
  f=dynpeak_smooth_IPI(y)
  IPI.data=[loc(2:$) y];
  tmed.data=[loc(2:$) f];
  thigh.data=[loc(2:$) f*(1+Beta)];      
  tlow.data=[loc(2:$) f*(1-Alpha)]; 
  if y<>[] then
    ax2.data_bounds(2,2)=max(max(f*(1+Beta)),max(y))
  end
  m2.data=[];
  m2.visible="off";
  drawnow()
endfunction
