// This file is part of the Dynpeak toolbox
// Copyright (C) 2012 - INRIA - Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
function params=lhpeaks_check_options(p,ns,fname)
  params=lhpeak_default_params();
  valid=["detectionthreshold","globalreltol" "localabstol" "threepointtol" "uncertainty" ...
         "period" "imposedpeaks" "ignoredpoints","selection"];
  if typeof(p)=="st" then
    fn=fieldnames(p)
    for k=1:size(fn,'*')
      select fn(k)
      case "detectionthreshold"
        detectionthreshold=p.detectionthreshold
        if type(detectionthreshold)<>1|~isreal(detectionthreshold)|size(detectionthreshold,'*')<>1 then
          error(msprintf(_("%s: Wrong type for field %s of input argument #%d: A real scalar expected.\n"),...
                         fname,"detectionthreshold",3))
        end
        if detectionthreshold<0 then
          error(msprintf(_("%s: Wrong value for field %s of input argument #%d: must be >= %d.\n"),...
                         fname,"detectionthreshold",3,0))
        end
        params.detectionthreshold=detectionthreshold;
 
      case "globalreltol" 
        globalreltol =p.globalreltol
        if type(globalreltol)<>1|~isreal(globalreltol)|size(globalreltol,'*')<>1 then
          error(msprintf(_("%s: Wrong type for field %s of input argument #%d: A real scalar expected.\n"),...
                         fname,"globalreltol",3))
        end
        if globalreltol<=0|globalreltol>1 then
          error(msprintf(_("%s: Wrong value for field %s of input argument #%d: Must be in the interval ]%d, %d].\n"),...
                         fname,"globalreltol",3,0,1))
        end
        params.globalreltol=globalreltol
      case "localabstol"
        localabstol=p.localabstol
        if type(localabstol)<>1|~isreal(localabstol)|size(localabstol,'*')<>1 then
          error(msprintf(_("%s: Wrong type for field %s of input argument #%d: A real scalar expected.\n"),...
                         fname,"localabstol",3))
        end
        if localabstol<=0 then
          error(msprintf(_("%s: Wrong value for field %s of input argument #%d: must be > %d.\n"),...
                         fname,"localabstol",3,0))
        end
        params.localabstol=localabstol
      case "threepointtol"
        threepointtol=p.threepointtol
        if type(threepointtol)<>1|~isreal(threepointtol)|size(threepointtol,'*')<>1 then
          error(msprintf(_("%s: Wrong type for field %s of input argument #%d: A real scalar expected.\n"),...
                         fname,"threepointtol",3))
        end
        if threepointtol<=0|threepointtol>1 then
          error(msprintf(_("%s: Wrong value for field %s of input argument #%d: Must be in the interval ]%d, %d].\n"),...
                         fname,"threepointtol",3,0,1))
        end
        params.threepointtol=threepointtol
      case "uncertainty"
        uncertainty=p.uncertainty
        if type(uncertainty)<>1|~isreal(uncertainty)|size(uncertainty,1)<>2 then
          error(msprintf(_("%s: Wrong type for field %s of input argument #%d: A  2 rows array expected.\n"),...
                         fname,"uncertainty",3))
        end
        if or(uncertainty(1,:)<0) then
          error(msprintf(_("%s: Wrong value for field %s of input argument #%d: first row elements must be positive.\n"),...
                         fname,"uncertainty",3))
        end
        if or(diff(uncertainty(1,:))<=0) then
          error(msprintf(_("%s: Wrong value for field %s of input argument #%d: Elements must be in increasing order.\n"),...
                         fname,"uncertainty",3))
        end
        if or(uncertainty(2,:)<0)|or(uncertainty(2,:)>1) then
          error(msprintf(_("%s: Wrong value for field %s of input argument #%d: first row elements must be in the interval [%d, %d].\n"),...
                         fname,"uncertainty",3,0,1))
        end
        params.uncertainty=uncertainty
      case "period"
        period=p.period
        if type(period)<>1|~isreal(period)|size(period,'*')<>1 then
          error(msprintf(_("%s: Wrong type for field %s of input argument #%d: A real scalar expected.\n"),...
                         fname,"period",3))
        end
        if period<=0 then
          error(msprintf(_("%s: Wrong value for field %s of input argument #%d: must be > %d.\n"),...
                         fname,"period",3,0))
        end
        params.period=period
      case "imposedpeaks"
        imposedpeaks=p.imposedpeaks
        if type(imposedpeaks)<>1|~isreal(imposedpeaks)|~(imposedpeaks==[]|or(size(imposedpeaks)==1)) then
          error(msprintf(_("%s: Wrong type for field %s of input argument #%d: [] or a vector  expected.\n"),...
                         fname,"imposedpeaks",3))
        end
        if imposedpeaks<>[]&(min(imposedpeaks)<1|max(imposedpeaks)>ns) then 
          error(msprintf(_("%s: Wrong value for field %s of input argument #%d: elements must be in [%d %d].\n"),...
                         fname,"imposedpeaks",3,1,ns))
        end
        params.imposedpeaks=imposedpeaks(:)
      case "ignoredpoints"
        ignoredpoints=p.ignoredpoints
        if type(ignoredpoints)<>1|~isreal(ignoredpoints)|~(ignoredpoints==[]|or(size(ignoredpoints)==1)) then
          error(msprintf(_("%s: Wrong type for field %s of input argument #%d: [] or a vector  expected.\n"),...
                         fname,"ignoredpoints",3))
        end
        if ignoredpoints<>[]&(min(ignoredpoints)<1|max(ignoredpoints)>ns) then 
          error(msprintf(_("%s: Wrong value for field %s of input argument #%d: elements must be in [%d %d].\n"),...
                         fname,"ignoredpoints",3,1,ns))
        end
          
        params.ignoredpoints=ignoredpoints(:)
      case "selection"
        selection=p.selection
        if selection<>[] then
          if type(selection)<>1|~isreal(selection)|size(selection,'*')<>2 then
            error(msprintf(_("%s: Wrong type for field %s of input argument #%d: [] or a vector with two elements expected.\n"),...
                           fname,"selection",3))
          end
          
          if selection(1)<1|selection(2)>ns then 
            error(msprintf(_("%s: Wrong value for field %s of input argument #%d: elements must be in [%d %d].\n"),...
                           fname,"selection",3,1,ns))
          end
          if selection(1)>=selection(2) then
            error(msprintf(_("%s: Wrong value for field %s of input argument #%d: second element must be greater than first one.\n"),...
                           fname,"selection",3))
          end
        end
          
        params.selection=selection(:)
      else
        error(msprintf(_("%s: Wrong value for input argument #%d: A struct with fields %s expected.\n"),...
                       fname,3,strcat(valid,",")))
      end
    end
  else
    error(msprintf(_("%s: Wrong value for input argument #%d: A struct with fields %s expected.\n"),...
                   fname,3,strcat(valid,",")))
  end
endfunction
