// This file is part of the Dynpeak toolbox
// Copyright (C) 2012 - INRIA - Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
function [p,mu] = polyfit(x,y,n,w)

  if size(x,'*')<>size(y,'*') then
    error(msprintf(_("%s: Wrong size for input arguments #%d and #%d: Same sizes expected.\n"),"polyfit",1,2))
  end
  if size(n,'*')<>1|n<>int(n) then
    error(msprintf(_("%s: Wrong type for input argument #%d: An int scalar expected.\n"),"polyfit",3))
  end
  if n<0 then
    error(msprintf(_("%s: Wrong value for input argument #%d: Must be >= %d expected.\n" ),"polyfit",3,0))
  end
  x = x(:);
  y = y(:);

  if argn(2)==4 then
    if size(x,'*')<>size(w,'*') then
      error(msprintf(_("%s: Wrong size for input arguments #%d and #%d: Same sizes expected.\n"),"polyfit",1,4))
    end
    w=w(:)
    y=w.*y
  end

  if argn(1) > 2
    mu = [mean(x); st_deviation(x)];
    x = (x - mu(1))/mu(2);
  end

  // Construct Vandermonde matrix.
  nx=size(x,'*');
  if argn(2)<4 then
    V(1:nx,1:n+1) = 1;
  else
    V(1:nx,1:n+1) = w*ones(1,n+1)
  end
  for j = n:-1:1
    V(:,j) = x.*V(:,j+1);
  end

  // Solve least squares problem.
  [Q,R] = qr(V,'e');
  p = R\(Q'*y);    // Same as p = V\y;
  if size(R,2) > size(R,1)
    warning(msprintf(_("%s: Polynomial is not unique; degree >= number of data points.\n","polyfit")))
    warning('polyfit: Polynomial is not unique; degree >= number of data points.')
  elseif cond(R) < 1.0e-10
    warning(msprintf(_("%s: Polynomial is badly conditioned. Remove repeated data points.\n","polyfit")))
  end
  r = y - V*p;
  p = p.';          // Polynomial coefficients are row vectors by convention.

endfunction

